@@ -13,7 +13,9 @@ from communication.sign_in import SignIn
 from communication.sign_up import SignUp
 from communication.change_password import ChgPsw
 from communication.forgot_password import ForgotPassword
-from communication.test_report import TestReport
+from communication.test_report_email import TestReportEmail
+from communication.get_report_email import GetReportEmail
+from communication.get_report import GetReport
 from define import *
 import main_paras
     
@@ -56,11 +58,23 @@ class CommunicationThread (threading.Thread):
         forgot_psw.join()
         print("forgot password done")
         
-    def testReport(self, token, email_1, email_2):        
-        test_report=TestReport(token, email_1, email_2)
-        test_report.start()
-        test_report.join()
-        print("test report done")
+    def testReportEmail(self, token, email_1, email_2):        
+        test_report_email=TestReportEmail(token, email_1, email_2)
+        test_report_email.start()
+        test_report_email.join()
+        print("test report email set done")
+
+    def getReportEmail(self, token):        
+        get_report_email=GetReportEmail(token)
+        get_report_email.start()
+        get_report_email.join()
+        print("get report email done")
+
+    def getReport(self, token):        
+        get_report=GetReport(token)
+        get_report.start()
+        get_report.join()
+        print("get report request sent")
 
     def webApi(self):
         while True:
@@ -81,8 +95,14 @@ class CommunicationThread (threading.Thread):
                 elif image[0] == FORGOT_PSW:    #[FORGOT_PSW, email]
                     self.forgotPsw(image[1])
                     continue
-                elif image[0] == TEST_REPORT:    #[TEST_REPORT, email_1, email_2]
-                    self.testReport(main_paras.sign_in_token, image[1], image[2])
+                elif image[0] == TEST_REPORT_EMAIL:    #[TEST_REPORT, email_1, email_2]
+                    self.testReportEmail(main_paras.sign_in_token, image[1], image[2])
+                    continue
+                elif image[0] == GET_REPORT_EMAIL:    #[GET_REPORT_EMAIL, passtoken]
+                    self.getReportEmail(main_paras.sign_in_token)
+                    continue
+                elif image[0] == GET_REPORT:    #[GET_REPORT, passtoken]
+                    self.getReport(main_paras.sign_in_token)
                     continue
             sendToApi(image, self.qForResult, self.passToken, [main_paras.test_place, main_paras.test_city, main_paras.test_country])
             self.qForCom.task_done()
