@@ -25,6 +25,9 @@ I2C_PORT                  =TOTAL_SLOTS
 #regular slot index is from 0 to TOTAL_SLOTS-1
 NON_SLOT_INDEX            =TOTAL_SLOTS
 SIGN_IN_FIRST_INDEX       = (NON_SLOT_INDEX+1)
+LANGUAGE_CHANGE_INDEX       = (NON_SLOT_INDEX+2)
+
+
 RESPONSE_SUCCESS   ='1'
 RESPONSE_ERROR     ='0'
 
@@ -92,6 +95,7 @@ Cassette_not_found           = 2
 Control_area_not_found       = 3
 Invalid_image_identifier     = 4
 Unexpected_exception         = 5
+Unknown                      = 6
 
 
 #  stacked slot index  #
@@ -155,8 +159,8 @@ ADDRESS =["finch","Toronto","Canada"]
 #@SIGNATURE = "client=laipac&signature=laipacws"
 #USER      = "userid=feng.gao@laipac.com&password=123456"
 SERIAL_NO = "654321"
-CASSETTE_DETECTION_MODE_AUTO   = 'AUTO MODE  '
-CASSETTE_DETECTION_MODE_MANUAL = 'MANUAL MODE'
+CASSETTE_DETECTION_MODE_AUTO   = 'auto mode'
+CASSETTE_DETECTION_MODE_MANUAL = 'manual mode'
 
 MAIN_PARA_CASSETTE_TYPE_CHIP = 'chip'
 MAIN_PARA_CASSETTE_TYPE_QR   = 'qr'
@@ -177,3 +181,6 @@ TEST_MODE_SPEED = 0
 TEST_MODE_NORMAL = 1
 
 popUp = None
+#trans = None
+
+LANGUAGE_LIST = ['English', 'Spanish', 'French', 'Japanese', 'Indonesian', 'Arabic', 'Thailand', 'Chinese Traditional']
