@@ -1,5 +1,7 @@
 import os
 from PyQt5 import QtCore, QtGui, QtWidgets
+from PyQt5.QtCore import QTranslator
+
 from PyQt5.uic import loadUi
 import pathlib
 import queue
@@ -23,6 +25,7 @@ from wifi_folder.wifi_list import _WifiList
 from volume.volume import _Volume
 from define import *
 from main_paras import setMainTopLeft, getMainTopLeft, mainChannelStart, setDetectionMode, getDetectionMode, setTestMode, getTestMode, queueForGui
+import main_paras
 from wrong_password import _WrongPassword
 from vkeyboard import handleVisibleChanged
 if sys.platform == 'linux':
@@ -51,6 +54,17 @@ class EmitThread(QtCore.QThread):
 class MainWindow(QtWidgets.QMainWindow):
     def __init__(self,parent=None, qForGui=None):
         super(MainWindow, self).__init__(parent)
+        self.trans=QTranslator()
+#        QtWidgets.QApplication.instance().removeTranslator(trans)
+        print( main_paras.info.show())
+        print('language: ', main_paras.info.getCurrentLanguage())        
+        language_qm = os.path.join(parentdir,main_paras.defaultLanguageFolder,main_paras.info.getCurrentLanguage(),'lng.qm')
+        #language_qm = os.path.join(parentdir,"en-jp.qm")
+        #print(language_qm)
+        self.trans.load(language_qm)
+        QtWidgets.QApplication.instance().installTranslator(self.trans)
+
+        
         emitThread = EmitThread(qForGui)
         emitThread.signal.connect(self.emitHook)
         emitThread.start()
@@ -63,6 +77,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.page=0
         self.slots = []
         self.allIcons = []
+        main_paras.setAllDefault()
         self.config()
         self.show()
         
@@ -81,6 +96,7 @@ class MainWindow(QtWidgets.QMainWindow):
                 
             except Exception as error:
                 print(error)
+                
         
         
     def config(self):
@@ -105,50 +121,24 @@ class MainWindow(QtWidgets.QMainWindow):
             #self.help.clicked.connect(self.helpHookForTest)
             
             mainChannelStart(self.mainChannelHook)
-            self.chip_mode.setText(getDetectionMode())
+            #self.chip_mode.setText(self.tr(getDetectionMode()))
             QtCore.QTimer.singleShot(0, self.setupAferUi)
             
             self.tempPolling.start(10000)
+
+
+            
+            
         except Exception as error:
             print(error)
         
-##    def config(self):
-##        try:
-##            slot_index = 1
-##            totalSlots = self.findChildren(QtWidgets.QStackedWidget)
-##            for slot in totalSlots:
-##                for page in range(1,6):
-##                    #print (page)
-##                    slot.setCurrentIndex(page)
-##                    style = '.QWidget{background-image:url(:/slot_' +str(slot_index)+ '/png/slot/slot_' +str(slot_index)+ '/' +pngName[page]+str(slot_index)+ '.png);border:0px}'
-##                    #print(style)
-##                    slot.currentWidget().setStyleSheet(style)
-##                slot.setCurrentIndex(0)
-##                slot_index+=1
-##                self.slots.append(Ui_Slot(slot))
-##
-##            self.allIcons = self.title.findChildren(QtWidgets.QToolButton)
-##            for icon in self.allIcons:
-##                icon.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
-##
-##            
-##            self.test_mode.clicked.connect(self.test_mode_hook)
-##
-##            self.help.clicked.connect(self.helpHookForTest)
-##            
-##            mainChannelStart(self.mainChannelHook)
-##            self.chip_mode.setText(getDetectionMode())
-##            #QtCore.QTimer.singleShot(0, self.setupAferUi)
-##        except Exception as error:
-##            print(error)
-
     def test_mode_hook(self):
         try:
             if self.test_mode.isChecked():
-                self.test_mode.setText(self.tr('Normal'))
+                self.test_mode.setText(self.tr('NORMAL'))
                 setTestMode(TEST_MODE_NORMAL)
             else:
-                self.test_mode.setText(self.tr(' Speed'))
+                self.test_mode.setText(self.tr('SPEED'))
                 setTestMode(TEST_MODE_SPEED)
         except Exception as e:
             print(e)
@@ -157,12 +147,12 @@ class MainWindow(QtWidgets.QMainWindow):
         try:
             print('main Channel', item)
             if   item == MAIN_PARA_CASSETTE_TYPE_CHIP:
-                self.chip_mode.setText(CASSETTE_DETECTION_MODE_AUTO)
+                self.chip_mode.setText(self.tr('AUTO MODE'))
                 setDetectionMode(CASSETTE_DETECTION_MODE_AUTO)
                 for each in self.slots:
                     each.setStatus(SLOT_STATUS_EMPTY, '')
             elif item == MAIN_PARA_CASSETTE_TYPE_QR:
-                self.chip_mode.setText(CASSETTE_DETECTION_MODE_MANUAL)
+                self.chip_mode.setText(self.tr('MANUAL MODE'))
                 setDetectionMode(CASSETTE_DETECTION_MODE_MANUAL)
                 for each in self.slots:
                     each.setStatus(SLOT_STATUS_SCAN, '')
@@ -185,9 +175,9 @@ class MainWindow(QtWidgets.QMainWindow):
             main_x, main_y =getMainTopLeft()
             self.help.setup(_HelpDialog, main_x, main_y)
             self.wifi.setup(_WifiList, main_x+205, main_y-7)
-            self.volume.setup(_Volume, main_x+245, main_y-7)
-            self.setting.setup(_SettingDialog, main_x+303, main_y-7)
-            self.profile_icon.setup(_ProfileDialog, main_x+335, main_y-7)
+            self.volume.setup(_Volume, main_x+255, main_y-7)
+            self.setting.setup(_SettingDialog, main_x+313, main_y-7)
+            self.profile_icon.setup(_ProfileDialog, main_x+345, main_y-7)
 
 ##            self.wifi.setup(_WifiList, main_x+225, main_y-7)
 ##            self.volume.setup(_Volume, main_x+275, main_y-7)
@@ -222,6 +212,10 @@ class MainWindow(QtWidgets.QMainWindow):
                 x,y = getMainTopLeft()
                 popUp.move(x,y)
                 popUp.show()
+
+            elif slotNo == LANGUAGE_CHANGE_INDEX:
+                print('LANGUAGE_CHANGE_INDEX')
+                self.language_change()
                 
             elif slotNo == NON_SLOT_INDEX:
                 for i in range(TOTAL_SLOTS):
@@ -239,14 +233,14 @@ class MainWindow(QtWidgets.QMainWindow):
                 self.slots[item[0]].setStatus(SLOT_STATUS_POSITIVE, qrCode)
             elif errCode == Negative_test_result:
                 self.slots[item[0]].setStatus(SLOT_STATUS_NEGATIVE, qrCode)
-            elif errCode == Control_area_not_found:
+            elif errCode == Invalid_image_identifier:
                 self.slots[item[0]].setStatus(SLOT_STATUS_INVALID, qrCode)
             elif errCode == DEVICE_STATE_CASSETTE_EMPTY:
                 self.slots[item[0]].setStatus(SLOT_STATUS_EMPTY, qrCode)
             else:
                 self.slots[item[0]].setStatus(SLOT_STATUS_WARNING, qrCode)
         except Exception as e:
-            print(e)
+            print('emitHook',e)
         
     def closeEvent(self,event):
         self.tempPolling.stop()
@@ -255,10 +249,22 @@ class MainWindow(QtWidgets.QMainWindow):
         
         print("Main window is closing")
 
+    def language_change(self):
+
+##        QtWidgets.QApplication.instance().removeTranslator(self.trans)
+##        trans.load(os.path.join(parentdir,"en-zh-cn.qm"))
+##        QtWidgets.QApplication.instance().installTranslator(trans)
+        self.close()
+        self.__init__(qForGui = queueForGui)
+        print('main gui repaint done')
+        
+        
+    
 if __name__ == "__main__":
-    from PyQt5.QtCore import QTranslator
     import sys
 
+    ##main_paras.languageInit()
+    
     print(QtWidgets.qApp)
 #     if(QtWidgets.qApp.focusWindow().isModal()):
 #         pass
@@ -266,13 +272,13 @@ if __name__ == "__main__":
     os.environ["QT_IM_MODULE"] = "qtvirtualkeyboard"
 
     
-##    trans=QTranslator()
-##    trans.load("gui.qm")
+#    trans=QTranslator()
+#    trans.load(os.path.join(parentdir,"en.qm"))
+#    trans.load(os.path.join(parentdir,"en-jp.qm"))
     
-    #qForGui=queue.Queue()
     app = QtWidgets.QApplication(sys.argv)
     QtGui.QGuiApplication.inputMethod().visibleChanged.connect(handleVisibleChanged)
-##    app.installTranslator(trans)
+#    app.installTranslator(trans)
     
     window=MainWindow(qForGui = queueForGui)
     rtn= app.exec_()
