@@ -7,7 +7,11 @@ import matplotlib.pyplot as plt
 import sys, os, inspect
 currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
 sys.path.insert(0, currentdir)
+parentdir = os.path.dirname(currentdir)
+sys.path.insert(0, parentdir)
 import calibration
+import define
+
 
 
 class Algorithm(ABC):
@@ -16,6 +20,8 @@ class Algorithm(ABC):
         self.v=v_list.copy()
         self.default=v_list.copy()
         self.originalImage=[]
+        self.simple = False
+        self.chart  = False
     def defaultVaules(self):
         return self.default
     def currentVaules(self):
@@ -30,7 +36,9 @@ class Algorithm(ABC):
             return self.v[index]
         return 0
         
-    def adjust(self,original,img,index=None, value=None):
+    def adjust(self,original,img,index=None, value=None, simple = False, chart = False):
+        self.simple = simple
+        self.chart  = chart
         plt_close()
         self.originalImage = original
         if index == None or value == None:
@@ -58,16 +66,16 @@ def plt_show(b=None, g=None, r=None, av=None):
 
     if b!=None:
         #print('show b')
-        plt.plot(x, b,  color='blue', label="Blue")
+        plt.plot(x, b,  color='blue', label="mean")
     if g!=None:
         #print('show g')
-        plt.plot(x, g,  color='green', label="Green")
+        plt.plot(x, g,  color='green', label="green")
     if r!=None:
         #print('show r')
-        plt.plot(x, r,  color='red', label="Red")
+        plt.plot(x, r,  color='red', label="Filtered mean")
     if av!=None:
         #print('show av')
-        plt.plot(x, av,  color='black', label="Ave")
+        plt.plot(x, av,  color='black', label="standard deviation")
     plt.xlabel("x")
     plt.ylabel("y")
     plt.ylim(0, 255)
@@ -77,10 +85,8 @@ def plt_show(b=None, g=None, r=None, av=None):
 def plt_close():
     plt.close()
     
-MIN_STDD = 20
-MAX_STDD = 25
 class Target(Algorithm):
-    def threshold(self, img):
+    def threshold(self, img, MAX_STDD, MIN_STDD):
         low = 0
         high = 255
         while True:
@@ -257,11 +263,6 @@ class Target(Algorithm):
         final_img = cv2.merge((b, g, r))
         return final_img
 
-    def gauss(self, image):
-        kernel = np.array([[0, -1, 0],[-1, 5, -1],[0, -1, 0]], np.float32)
-        result = cv2.filter2D(image, -1, kernel=kernel)
-        return result
-
     def color_zhi_fang(self, image):
 
         b, g, r = cv2.split(image)
@@ -282,9 +283,15 @@ class Target(Algorithm):
     def clahe(self, image):
 
         b, g, r = cv2.split(image)
+#        clahe = cv2.createCLAHE(clipLimit=29.0, tileGridSize=(1,13))
+
+#        clahe = cv2.createCLAHE(clipLimit=29.0, tileGridSize=(11,21))
+
+        clahe = cv2.createCLAHE(clipLimit=45.0, tileGridSize=(9,40))
+        
 
 #        clahe = cv2.createCLAHE(clipLimit=29.0, tileGridSize=(1,6))
-        clahe = cv2.createCLAHE(clipLimit=29.0, tileGridSize=(1,13))
+
         #clahe = cv2.createCLAHE(clipLimit=0.0, tileGridSize=(13,13))
         #clahe = cv2.createCLAHE()
         
@@ -298,14 +305,13 @@ class Target(Algorithm):
         return result
 #horizontalStack = np.concatenate((self.img, self.newImg), axis=1)
 
-    def edsr(self,img):
-#        result = cv2.resize(img,dsize=None,fx=4,fy=4)
-        sr = cv2.dnn_superres.DnnSuperResImpl_create()
-        path = os.path.join(currentdir,"EDSR_x4.pb")
-        sr.readModel(path)
-        sr.setModel("edsr", 4) # set the model by passing the value and the upsampling ratio
-        result = sr.upsample(img) # upscale the input image
-        return result
+    def detect(self,img):
+        hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
+        #Red color rangle  169, 100, 100 , 189, 255, 255
+        lower_range = np.array([110,50,50])
+        upper_range = np.array([130,255,255])
+        mask = cv2.inRange(hsv, lower_range, upper_range)
+        return mask
 
     def filt(self, original_list, adjust):
         average = sum(original_list)/len(original_list)
@@ -325,23 +331,26 @@ class Target(Algorithm):
         return rtn_list
             
 
-    def filt_width(self, original_list, number,std_list, max_std):
+    def filt_width(self, original_list, number,std_list, max_std, min_diff):
         sort_list=original_list.copy()
         sort_list.sort()
 
         av_std=sum(std_list)/len(std_list)
         
         average = sort_list[number-1]
+        #average = sum(original_list)/len(original_list)
+
+        
         maxim=sort_list[-1]
         
         rtn_list = []
         for each in original_list:
-            if each>average:
+            if each>average : ## and (each - average) >10:
                 rtn_list.append(maxim)
-##            elif average-each <0:
-##                pass #rtn_list.append(maxim)
             elif (std_list[original_list.index(each)]-av_std)>max_std and std_list[original_list.index(each)]>30:
                 pass
+            elif (average - each) < min_diff:
+                pass
             else:
                 rtn_list.append(average)
         #print('rtn_list', rtn_list)
@@ -410,28 +419,6 @@ class Target(Algorithm):
         return rtn, std
 
 
-##    def mean_std(self, image, i):
-##        height, width = image.shape[:2]
-##        top    = image[0:int(height/3), 0:width]
-##        mid    = image[int(height/3):int(height*2/3), 0:width]
-##        bottom = image[int(height*2/3):height, 0:width]
-##
-##        #print(top.shape[:2],mid.shape[:2],bottom.shape[:2])
-##        
-##        top_mn, top_std = self.block_mean_std(top,i)
-##        mid_mn, mid_std = self.block_mean_std(mid,i)        
-##        btm_mn, btm_std = self.block_mean_std(bottom,i)
-##
-##
-##        mn_list = [top_mn, mid_mn, btm_mn]
-##        max_mn = max(mn_list)
-##        m_index = mn_list.index(max_mn)
-##
-##        std = [top_std, mid_std, btm_std][m_index]
-##
-##        return max_mn, std
-
-
     def mean_std(self, image, i):
         num =1
         height, width = image.shape[:2]
@@ -489,7 +476,8 @@ class Target(Algorithm):
     def lightBarConfirm(self, mn_list, deep, width):
         value = min(mn_list)
         if value == max(mn_list):
-            return False
+            #print('Negative')
+            return define.Negative_test_result
         length = []
         count = 0
         for each in mn_list:
@@ -504,151 +492,129 @@ class Target(Algorithm):
         
         max_width = max(length)
         #print(max_width, length)
-        if value > deep and max_width > width:
-            return True
-        return False
-    
+#        if value > deep and max_width > width:
+        if max_width > width:
+            #print('Positive')
+            return define.Positive_test_result
+        #print('Negative')
+        return define.Negative_test_result
+##    
+##MIN_STDD = 20
+##MAX_STDD = 25
 
         
-##    def barIdt(self, image, offset, width):
-##        reduce =0
-##        height, _ = image.shape[:2]
-##        h_h = int(height/2)
-##        
-##        mid       = image[0: h_h,
-##                            offset: offset+width]                            
-##        left      = image[0: h_h,
-##                            offset-(width-reduce): offset]                            
-##        right     = image[0: h_h,
-##                            offset+width: offset+width+(width - reduce)]
-##
-##        up_mn    =[self.av_mean(left), self.av_mean(mid), self.av_mean(right)]
-##        up_block =[left,mid,right]
-##        print(up_mn)
-##        
-##        mid       = image[h_h: height,
-##                            offset: offset+width]                            
-##        left      = image[h_h: height,
-##                            offset-(width-reduce): offset]                            
-##        right     = image[h_h: height,
-##                            offset+width: offset+width+(width - reduce)]
-##        down_mn    =[self.av_mean(left), self.av_mean(mid), self.av_mean(right)]
-##        down_block =[left,mid,right]
-##        print(down_mn)
-##
-##        rtn =False
-##        if up_mn[1] == min(up_mn) and down_mn[1] == min(down_mn):
-##            if(min([up_mn[1], down_mn[1]])>90):
-##                rtn =True
-##        return rtn , [left,mid,right]
-        
     def calculation(self, image):
-        print('original',image.shape[:2])
+        #print('original',image.shape[:2])
         gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
-        thresh, std = self.threshold(gray)
+        thresh, std = self.threshold(gray, 25, 20)
         #print('target std',std)
         kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (8,8))
         morph = cv2.morphologyEx(thresh, cv2.MORPH_OPEN, kernel)
         height,width = morph.shape[:2]
-        
-        area = {
-            'x' : 0,
-            'y' : 0,
-            'width' : width,
-            'height': height
-            }
-        #print('area', area)
-        target_area=self.areaFilter(morph, area, [calibration.IDEAL_BAR_HEIGHT, calibration.IDEAL_BAR_WIDTH])
-        print('idea bar col, row', calibration.IDEAL_BAR_COL_OFFSET, calibration.IDEAL_BAR_ROW_OFFSET)
 
-        target_crop =morph[target_area['y']:target_area['y']+target_area['height'],
-                           target_area['x']:target_area['x']+target_area['width']]
-
-        
-
-        
-        mean, stdev =cv2.meanStdDev(target_crop)
-
-        mn = int(mean[0][0])
-        #print('target mean std',mn,std)
-        blank = False
-        if mn >  115: ## Blank cassette
-            m_x =0
-            m_y =0
-            blank = True
-            print('blank')
+        if(height == calibration.FINAL_HEIGHT and width == calibration.FINAL_WIDTH): ## For those pictures from server or dash board
+            blc = image
         else:
-            m_x = target_area['x'] - calibration.IDEAL_BAR_COL_OFFSET
-            m_y = target_area['y'] - calibration.IDEAL_BAR_ROW_OFFSET
-            cal = calibration.cr_modified(self.originalImage, m_y, m_x)
-            cal_h, cal_w = cal.shape[:2]
-            print(cal_h,cal_w)
-            if cal_h != calibration.FINAL_HEIGHT or cal_w != calibration.FINAL_WIDTH: #out of range, wrong bar position detection
-                print('wrong bar position')
+            area = {
+                'x' : 0,
+                'y' : 0,
+                'width' : width,
+                'height': height
+                }
+            #print('area', area)
+            target_area=self.areaFilter(morph, area, [calibration.IDEAL_BAR_HEIGHT, calibration.IDEAL_BAR_WIDTH])
+            #print('idea bar col, row', calibration.IDEAL_BAR_COL_OFFSET, calibration.IDEAL_BAR_ROW_OFFSET)
+            #print('target_area', target_area)
+     
+            target_crop =morph[target_area['y']:target_area['y']+target_area['height'],
+                               target_area['x']:target_area['x']+target_area['width']]
+
+        
+            mean, stdev =cv2.meanStdDev(target_crop)
+
+            mn = int(mean[0][0])
+        
+            blank = False
+            if mn >  115: ## Blank cassette
+                m_x =0
+                m_y =0
                 blank = True
+                #print('blank')
+            else:
+                m_x = target_area['x'] - calibration.IDEAL_BAR_COL_OFFSET
+                m_y = target_area['y'] - calibration.IDEAL_BAR_ROW_OFFSET
+                cal = calibration.cr_modified(self.originalImage, m_y, m_x)
+                cal_h, cal_w = cal.shape[:2]
+                #print(cal_h,cal_w)
+                if cal_h != calibration.FINAL_HEIGHT or cal_w != calibration.FINAL_WIDTH: #out of range, wrong bar position detection
+                    #print('wrong bar position')
+                    blank = True
 
-        if blank:
-            final = calibration.cr_modified(self.originalImage, 0, 0)
-            return [final]
+            if blank:
+                final = calibration.cr_modified(self.originalImage, 0, 0)
+                #print('Invalid')
+                return [final, str(define.Invalid_image_identifier)]
 
-        cal = calibration.cr_modified(self.originalImage, m_y, m_x)
-        blc = self.balance(cal,198)
-
-
-        if sys.platform != 'win32':   ## balancing only 
-            return[blc]
-
-        
+            cal = calibration.cr_modified(self.originalImage, m_y, m_x)
+            blc = self.balance(cal,198)
+            
+        #if sys.platform != 'win32':   ## balancing only 
+        if self.simple == True:        ## balancing only
+            return[blc, str(define.Unknown)]
         clh = self.clahe(blc)
+        #clh = self.balance(clh,168)
 
         
-
         secondBarToFirst = 160
         secondBarWidth = calibration.IDEAL_BAR_WIDTH+4
 
-        reduce = 10
-        weak_height = calibration.IDEAL_BAR_HEIGHT -reduce*2
-        weak_row_start = calibration.FINAL_BAR_ROW + reduce        
+        reduce = 15
+        weak_height = calibration.IDEAL_BAR_HEIGHT -reduce*2+10
+        weak_row_start = calibration.FINAL_BAR_ROW + (reduce -8)        
 
         block_offset =142
-        block_width  = calibration.IDEAL_BAR_WIDTH*2+ int(calibration.IDEAL_BAR_WIDTH/2)
+        block_width  = calibration.IDEAL_BAR_WIDTH*3 #calibration.IDEAL_BAR_WIDTH*2+ int(calibration.IDEAL_BAR_WIDTH/2)
         sample_width = 1#int(calibration.IDEAL_BAR_WIDTH/16)
         block = clh[ weak_row_start: weak_row_start+weak_height,
                               calibration.FINAL_BAR_COL+block_offset : calibration.FINAL_BAR_COL+block_offset + block_width]
+
+        nice_block = blc[ weak_row_start: weak_row_start+weak_height,
+                              calibration.FINAL_BAR_COL+block_offset : calibration.FINAL_BAR_COL+block_offset + block_width]
+
+
+##        second_blc=self.balance(clh,198)
+##        return [blc ,clh, second_blc]
         
         b = 0
         g = 1
         r = 2
         av =3
 
-        mn_list, std_list = self. av_mean_draw(block, sample_width, g)
+        mn_list, std_list = self. av_mean_draw(block, sample_width, av)
 
-        #filt_list  = self.filt(mn_list, -29)
-        #std_list = self.filt(std_list)
+
+##        std_filter = 6
+##        mn_filter  = 100
+##        width_filter = 13      # 0.0, 13,13
+##        number_of_low_mn = 26#30 #calibration.IDEAL_BAR_WIDTH
 
         std_filter = 6
         mn_filter  = 100
-        width_filter = 10      # 0.0, 13,13
-        number_of_low_mn = 20#30 #calibration.IDEAL_BAR_WIDTH
+        width_filter = 16     # 0.0, 13,13
+        number_of_low_mn = 36#30 #calibration.IDEAL_BAR_WIDTH
+        min_diff   = 5
         
 
-##        std_filter = 50
-##        mn_filter  = 100
-##        width_filter = 10      # 22, 5,19
-##        number_of_low_mn = 30 #calibration.IDEAL_BAR_WIDTH
-
-
-        #filt_width_list_no_std = self.filt_width_no_std(mn_list, number_of_low_mn)
-
-        filt_width_list = self.filt_width(mn_list, number_of_low_mn, std_list, std_filter)
-        if sys.platform == 'win32':
+        filt_width_list = self.filt_width(mn_list, number_of_low_mn, std_list, std_filter, min_diff)
+#        if sys.platform == 'win32':
+        if self.chart == True:
             plt_show(av=std_list, b=mn_list)
             plt_show(r=filt_width_list)
         
 
 
         rtn = self.lightBarConfirm(filt_width_list, mn_filter, width_filter)
-        print(rtn)
+        
         final = blc
 ##        firstBar = final[calibration.FINAL_BAR_ROW: calibration.FINAL_BAR_ROW+calibration.IDEAL_BAR_HEIGHT,
 ##                       calibration.FINAL_BAR_COL: calibration.FINAL_BAR_COL+calibration.IDEAL_BAR_WIDTH]
@@ -658,7 +624,26 @@ class Target(Algorithm):
 ##            final[calibration.FINAL_BAR_ROW: calibration.FINAL_BAR_ROW+calibration.IDEAL_BAR_HEIGHT,
 ##                           calibration.FINAL_BAR_COL+secondBarToFirst: calibration.FINAL_BAR_COL+ secondBarToFirst+calibration.IDEAL_BAR_WIDTH] = firstBar
 ##        
-        return [final, clh, block]
+
+
+        return [final, str(rtn) ,clh, block]
+
+        nice_gray = cv2.cvtColor(block, cv2.COLOR_BGR2GRAY)
+        nice_thresh, std = self.threshold(nice_gray, 100, 80)
+        #print('target std',std)
+        nice_kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (8,8))
+        nice_morph = cv2.morphologyEx(nice_thresh, cv2.MORPH_OPEN, nice_kernel)
+        return [final, str(rtn) ,clh, block, nice_morph]
+        
+        
+            
+        r =80
+        h   =160 
+        top = final[r:r+h,0:width]
+        bottom = clh[r:r+h,0:width]
+        combine = np.concatenate((top, bottom), axis=0)
+        return [combine, str(rtn) ,clh, block]
+        
 
 
 
@@ -978,6 +963,17 @@ class EDSR(Algorithm):
         return [result]
 
 
+class Detection(Algorithm):
+    def calculation(self,img):
+        hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
+        #Red color rangle  169, 100, 100 , 189, 255, 255
+        lower_range = np.array([self.v[0],self.v[1],self.v[2]])
+        upper_range = np.array([self.v[3],self.v[4],self.v[5]])
+        mask = cv2.inRange(hsv, lower_range, upper_range)
+        return [mask]
+
+
+
 nothing = Nothing('Original',[]) 
 gauss = Gauss('gauss', [-1, 1, -1, 2, 6, 2, 1, 1, -6, 9])
 contrast = Contrast('conrast',[1,0])
@@ -1008,3 +1004,6 @@ he_ycrcb = HE_ycrcb('he_ycrcb',[])
 bilateral = Bilateral('bilateral',[1])
 
 edsr = EDSR('edsr',[])
+
+detection = Detection('detection',[126,92,246,176,92,246])
+
